clearvars -except timerS own_machine_performance; clc; close all;

%% Parameters
%CRRA utility with parameter gamma
ga = 2; 
r = 0.03; %interest rate
rho=1/.95-1; %discount rate


%ORNSTEIN-UHLENBECK PROCESS dlog(z) = -the*log(z)dt + sig2*dW
%STATIONARY DISTRIBUTION IS log(z) ~ N(0,Var) WHERE Var = sig2/(2*the)
Corr = 0.95; 
the = -log(Corr);
Var = 0.2^2;
sig2 = Var*2*the; %continuous-time analogue of Var*(1-rho^2)
logzmean = 0;
zmean = logzmean + exp(Var/2); %MEAN OF LOG-NORMAL DISTRIBUTION N(0,Var)

J=9;
%zmax = 99th percentile of log-normal distribution:
z_stan_99 = 2.33; z_stan_975 = 1.96; z_stan_95 = 1.6449; z_stan_999 = 3.09; z_stan_9999 = 3.71;
logzmax = z_stan_99*sqrt(Var);
zmax = exp(logzmax);
zmin = exp(-logzmax);
z = linspace(zmin,zmax,J);
z=z/zmean;
dz = (zmax-zmin)/(J-1);
dz2 = dz^2;

double C; double duration; double duration_dist; double duration_agg;

I = 10000;
psi=0.0;
amin = -psi*zmin/r; %borrowing constraint
amax=50;

maxit= 1000;
crit = 1e-6;
Delta = 10;

fname= 'cont_true_dist.txt'; %name of the file containing the 'true' consumption policy
aggconsname= 'agg_c_cont.txt'; %name of the file to store the "true" aggregate consumption

agname= 'cont_agrid.txt'; %name of the file containing the asset grid points
igname= 'cont_igrid.txt'; %name of the file containing the asset grid points

%% (non-uniform) Grid
coeff = 0; power=2;
x = linspace(0,1,I).^power';
%xx  = x + coeff*x.^power;
%xmax = max(xx); xmin = min(xx);
a = amin+(amax-amin)*x;
daf = ones(I,1);
dab = ones(I,1);
daf(1:I-1) = a(2:I)-a(1:I-1);
dab(2:I) = a(2:I)-a(1:I-1);
daf(I)=daf(I-1); dab(1)=dab(2);


mu = (-the*log(z) + sig2/2).*z; %DRIFT (FROM ITO'S LEMMA)
s2 = sig2.*z.^2; %VARIANCE (FROM ITO'S LEMMA)

aa = a*ones(1,J);
daaf = daf*ones(1,J);
daab = dab*ones(1,J);
zz = ones(I,1)*z;

Vaf = zeros(I,J);
Vab = zeros(I,J);
Vzf = zeros(I,J);
Vzb = zeros(I,J);
Vzz = zeros(I,J);
c = zeros(I,J);

%CONSTRUCT MATRIX Aswitch SUMMARIZING EVOLUTION OF z
yy = - s2/dz2 - mu/dz;
chi =  s2/(2*dz2);
zeta = mu/dz + s2/(2*dz2);

%This will be the upperdiagonal of the B_switch
updiag=zeros(I,1); %This is necessary because of the peculiar way spdiags is defined.
for j=1:J
    updiag=[updiag;repmat(zeta(j),I,1)];
end

%This will be the center diagonal of the B_switch
centdiag=repmat(chi(1)+yy(1),I,1);
for j=2:J-1
    centdiag=[centdiag;repmat(yy(j),I,1)];
end
centdiag=[centdiag;repmat(yy(J)+zeta(J),I,1)];

%This will be the lower diagonal of the B_switch
lowdiag=repmat(chi(2),I,1);
for j=3:J
    lowdiag=[lowdiag;repmat(chi(j),I,1)];
end

%Add up the upper, center, and lower diagonal into a sparse matrix
Aswitch=spdiags(centdiag,0,I*J,I*J)+spdiags(lowdiag,-I,I*J,I*J)+spdiags(updiag,I,I*J,I*J);

%% Initial Guess
v0 = (zz + r.*aa).^(1-ga)/(1-ga)/rho;
v = v0;

%% Iteration
tic;
for n=1:maxit
    V = v;
    % forward difference
    Vaf(1:I-1,:) = (V(2:I,:)-V(1:I-1,:))./(aa(2:I,:) - aa(1:I-1,:));
    Vaf(I,:) = (max(z + r.*amax,10^(-10))).^(-ga); %will never be used, but impose state constraint a<=amax just in case
    % backward difference
    Vab(2:I,:) = (V(2:I,:)-V(1:I-1,:))./(aa(2:I,:) - aa(1:I-1,:));
    Vab(1,:) = (max(z + r.*amin,10^(-10))).^(-ga); %state constraint boundary condition

    I_concave = Vab > Vaf; %indicator whether value function is concave (problems arise if this is not the case)

    %consumption and savings with forward difference
    cf = max(Vaf,10^(-10)).^(-1/ga);
    sf = zz + r.*aa - cf;
    %consumption and savings with backward difference
    cb = max(Vab,10^(-10)).^(-1/ga);
    sb = zz + r.*aa - cb;
    %consumption and derivative of value function at steady state
    c0 = zz + r.*aa;

    % dV_upwind makes a choice of forward or backward differences based on
    % the sign of the drift
    If = sf > 0; %positive drift --> forward difference
    Ib = sb < 0; %negative drift --> backward difference
    I0 = (1-If-Ib); %at steady state
 
    %STATE CONSTRAINT at amin: USE BOUNDARY CONDITION UNLESS sf > 0:
    %already taken care of automatically

    c = cf.*If + cb.*Ib + c0.*I0;
    u = c.^(1-ga)/(1-ga);

    %CONSTRUCT MATRIX A
    X = -min(sb,0)./daab;
    Y = -max(sf,0)./daaf + min(sb,0)./daab;
    Z = max(sf,0)./daaf;

    updiag=0; %This is needed because of the peculiarity of spdiags.
    for j=1:J
        updiag=[updiag;Z(1:I-1,j);0];
    end

    centdiag=reshape(Y,I*J,1);

    lowdiag=X(2:I,1);
    for j=2:J
        lowdiag=[lowdiag;0;X(2:I,j)];
    end

    AA =spdiags(centdiag,0,I*J,I*J)+spdiags([updiag;0],1,I*J,I*J)+spdiags([lowdiag;0],-1,I*J,I*J);

    A = AA + Aswitch;
    B = (1/Delta + rho)*speye(I*J) - A;

    u_stacked = reshape(u,I*J,1);
    V_stacked = reshape(V,I*J,1);

    b = u_stacked + V_stacked/Delta;

    V_stacked = B\b; %SOLVE SYSTEM OF EQUATIONS

    V = reshape(V_stacked,I,J);

    Vchange = V - v;
    v = V;

    dist(n) = max(max(abs(Vchange)));
    %disp(['Iteration no. ' int2str(n), ' convergence criterion ' num2str(dist(n))]);

    if dist(n)<crit
        %disp('Value Function Converged, Iteration = ')
        disp(n)
        break
    end
end
duration=toc;


%% FOKKER-PLANCK EQUATION %
%WORK WITH RESCALED DENSITY \tilde{g} BELOW
tic;
da_tilde = 0.5*(dab + daf);
da_tilde(1) = 0.5*daf(1); da_tilde(I) = 0.5*dab(I);
da_stacked = repmat(da_tilde,J,1);
grid_diag = spdiags(da_stacked,0,I*J,I*J);

AT = A';
b = zeros(I*J,1);

%need to fix one value, otherwise matrix is singular
i_fix = 1;
b(i_fix)=.1;
row = [zeros(1,i_fix-1),1,zeros(1,I*J-i_fix)];
AT(i_fix,:) = row;

%Solve linear system
g_tilde = AT\b;

%rescale \tilde{g} so that it sums to 1
g_sum = g_tilde'*ones(I*J,1);
g_tilde = g_tilde./g_sum;

gg = grid_diag\g_tilde; %convert from \tilde{g} to g

g = reshape(gg,I,J);

c_stacked = reshape(c,I*J,1);
C = (c_stacked.*gg)'*da_stacked;

duration_dist = toc;
duration_agg = duration_dist + duration;
%% Write True Values to File

true_vals = c;
p = '../Data/';
fname=strcat(p,fname);
csvwrite(fname,true_vals);

aggconsname = strcat(p, aggconsname);
csvwrite(aggconsname,[C;duration;duration_agg]);

agname=strcat(p,agname);
igname= strcat(p, igname);
csvwrite(agname,aa);
csvwrite(igname,zz);
