clearvars -except timerS own_machine_performance; clc; close all;

tic;

theta = 1;
r = 0;
rho = 0.05;
w = .1;

I=1000;
amin = 0;
amax = 1;
a = linspace(amin,amax,I)';
da = (amax-amin)/(I-1); 

maxit=10000;
crit = 10^(-10);

Delta = 1000;

dVf = zeros(I,1);
dVb = zeros(I,1);
c = zeros(I,1);

%INITIAL GUESS
v0 = -exp(-theta*(w + 0.01.*a))/(theta*rho);
v = v0;

for n=1:maxit
    V = v;
    % forward difference
    dVf(1:I-1) = (V(2:I)-V(1:I-1))/da;
    dVf(I) = exp(-theta*(w + r.*amax)); %will never be used, but impose state constraint a<=amax just in case
    % backward difference
    dVb(2:I) = (V(2:I)-V(1:I-1))/da;
    dVb(1) = exp(-theta*(w + r.*amin)); %state constraint boundary condition
       
    %consumption and savings with forward difference
    cf = -log(dVf)/theta;
    ssf = w + r.*a - cf;
    %consumption and savings with backward difference
    cb = -log(dVb)/theta;
    ssb = w + r.*a - cb;
    %consumption and derivative of value function at steady state
    c0 = w + r.*a;
    dV0 = exp(-theta*c0);
    
    % dV_upwind makes a choice of forward or backward differences based on
    % the sign of the drift    
    If = ssf > 0; %positive drift --> forward difference
    Ib = ssb < 0; %negative drift --> backward difference
    I0 = (1-If-Ib); %at steady state
    
    c = cf.*If + cb.*Ib + c0.*I0; %important to include third term
    
    u = -exp(-theta*c)/theta;

    %CONSTRUCT MATRIX
    X = -min(ssb,0)/da;
    Y = -max(ssf,0)/da + min(ssb,0)/da;
    Z = max(ssf,0)/da;
    
    %sparse matrix: faster
    A =spdiags(Y,0,I,I)+spdiags(X(2:I),-1,I,I)+spdiags([0;Z(1:I-1)],1,I,I);
    B = (rho + 1/Delta)*speye(I) - A;
    
    b = u + V/Delta;
    V = B\b; %SOLVE SYSTEM OF EQUATIONS
    Vchange = V - v;
    v = V;
    dist(n) = max(abs(Vchange));
    if dist(n)<crit
        disp('Value Function Converged, Iteration = ')
        disp(n)
        break
    end
end
toc;

adot = w + r.*a - c;

nu = rho/theta;
c_cf = w + sqrt(2*nu.*a);
s_cf = - sqrt(2*nu.*a);

%Plot sizes
FigureSize = [20 16];

plot(a,V,'LineWidth',2)
set(gca,'FontSize',12)
grid
xlabel('a')
ylabel('V(a)')
xlim([amin amax])

plot(a,adot,a,s_cf,a,zeros(1,I),'--','LineWidth',2)
set(gca,'FontSize',14)
grid
xlabel('a')
ylabel('s(a)')
xlim([amin amax])

figure(1)
plot(a,c,a,c_cf,'--','LineWidth',2)
set(gca,'FontSize',18)
legend({'Closed-form solution $c(a)$', ...
    'Numerical solution, $\hat{c}(a)$'}, ...
    'Interpreter','latex','Location','SouthEast')
xlabel('Wealth $a$','Interpreter','latex')
ylabel('Consumption','Interpreter','latex')
xlim([amin amax])
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure12a.eps


c_error = 100*(c - c_cf)./c_cf;
figure(2)
plot(a,c_error,a,zeros(I,1),'k--','LineWidth',2)
set(gca,'FontSize',18)
xlabel('Wealth, $a$','Interpreter','latex')
ylabel('Percentage Error, $100 \times (\hat{c}(a) - c(a))/c(a)$','Interpreter','latex')
xlim([-0.1 amax])
ylim([-0.4 0.05])
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure12b.eps


