clearvars -except timerS own_machine_performance; clc; close all;


tic;

s = 2;
rho = 0.05;
r = 0.025;
r_borr = 0.075;
z1 = .1;
z2 = .15;
z3 = .2;
J=3;

if J==2;
    la_mat = [-1 1; 1 -1];
    z = [z1,z2];
elseif J==3
    z = [z1,z2,z3];
    factor = 0.03;
    la_mat = factor*[-2, 1, 1; 1, -2, 1; 1, 1, -2];
end

I=1000;
amin_natural = z1/r_borr
amin = -0.5;
amax = 2;
a = linspace(amin,amax,I)';
da = (amax-amin)/(I-1);

aa = a*ones(1,J);
zz = ones(I,1)*z;


maxit= 100;
crit = 10^(-6);
Delta = 1000;

dVf = zeros(I,J);
dVb = zeros(I,J);
c = zeros(I,J);

Aswitch = kron(la_mat,speye(I));

rr = r.*(aa>=0) + r_borr.*(aa<0);

v0 = (zz + max(r,0.01).*aa).^(1-s)/(1-s)/rho;

v = v0;

%maxit=2;
for n=1:maxit
    V = v;
    V_n(:,:,n)=V;
    % forward difference
    dVf(1:I-1,:) = (V(2:I,:)-V(1:I-1,:))/da;
    dVf(I,:) = (z + r.*amax).^(-s); %will never be used
    % backward difference
    dVb(2:I,:) = (V(2:I,:)-V(1:I-1,:))/da;
    dVb(1,:) = (z + r_borr.*amin).^(-s); %state constraint boundary condition
    
    I_concave = dVb > dVf; %indicator whether value function is concave (problems arise if this is not the case)
    
    %consumption and savings with forward difference
    cf = max(dVf,10^(-10)).^(-1/s);
    ssf = zz + rr.*aa - cf;
    Hf = cf.^(1-s)/(1-s) + dVf.*ssf;
    %consumption and savings with backward difference
    cb = max(dVb,10^(-10)).^(-1/s);
    ssb = zz + rr.*aa - cb;
    Hb = cb.^(1-s)/(1-s) + dVb.*ssb;
    
    %consumption and derivative of value function at steady state
    c0 = zz + rr.*aa;
    H0 = c0.^(1-s)/(1-s);
    
    Ineither = (1-(ssf>0)) .* (1-(ssb<0));
    Iunique = (ssb<0).*(1-(ssf>0)) + (1-(ssb<0)).*(ssf>0);
    Iboth = (ssb<0).*(ssf>0);
    Ib = Iunique.*(ssb<0) + Iboth.*(Hb>=Hf);
    If = Iunique.*(ssf>0) + Iboth.*(Hf>=Hb);
    I0 = Ineither;
    
    c = cf.*If + cb.*Ib + c0.*I0;
    u = c.^(1-s)/(1-s);
    
    %CONSTRUCT MATRIX
    X = -Ib.*ssb/da;
    Y = -If.*ssf/da + Ib.*ssb/da;
    Z = If.*ssf/da;

    if J==2;
        A1=spdiags(Y(:,1),0,I,I)+spdiags(X(2:I,1),-1,I,I)+spdiags([0;Z(1:I-1,1)],1,I,I);
        A2=spdiags(Y(:,2),0,I,I)+spdiags(X(2:I,2),-1,I,I)+spdiags([0;Z(1:I-1,2)],1,I,I);
        A = [A1,sparse(I,I);sparse(I,I),A2] + Aswitch;
    elseif J==3
        A1=spdiags(Y(:,1),0,I,I)+spdiags(X(2:I,1),-1,I,I)+spdiags([0;Z(1:I-1,1)],1,I,I);
        A2=spdiags(Y(:,2),0,I,I)+spdiags(X(2:I,2),-1,I,I)+spdiags([0;Z(1:I-1,2)],1,I,I);
        A3=spdiags(Y(:,3),0,I,I)+spdiags(X(2:I,3),-1,I,I)+spdiags([0;Z(1:I-1,3)],1,I,I);
        A = [A1,sparse(I,I),sparse(I,I);sparse(I,I),A2,sparse(I,I);sparse(I,I),sparse(I,I),A3] + Aswitch;
    end
    
    B = (1/Delta + rho)*speye(J*I) - A;

    u_stacked = reshape(u,I*J,1);
    V_stacked = reshape(V,I*J,1);
    
    b = u_stacked + V_stacked/Delta;
    V_stacked = B\b; %SOLVE SYSTEM OF EQUATIONS
    
    V = reshape(V_stacked,I,J);
    
    Vchange = V - v;
    v = V;

    dist(n) = max(max(abs(Vchange)));
    if dist(n)<crit
        disp('Value Function Converged, Iteration = ')
        disp(n)
        break
    end
end

toc;

%%%%%%%%%%%%%%%%%%%%%%%%%%
% FOKKER-PLANCK EQUATION %
%%%%%%%%%%%%%%%%%%%%%%%%%%
AT = A';
b = zeros(J*I,1);

%need to fix one value, otherwise matrix is singular
i_fix = 200;
b(i_fix)=.1;
row = [zeros(1,i_fix-1),1,zeros(1,J*I-i_fix)];
AT(i_fix,:) = row;

%Solve linear system
gg = AT\b;
g_sum = gg'*ones(J*I,1)*da;
gg = gg./g_sum;
g = reshape(gg,I,J);

adot = zz + rr.*aa - c;

%Plot sizes
FigureSize = [20 16];

figure(1)
hold off
amax1 = 0.6;
amin1 = amin-0.03;
h1 = plot(a,adot(:,1),'g--',a,adot(:,2),'b',a,adot(:,3),'r-.','LineWidth',2)
legend(h1,{'$s_0(a)$','$s_1(a)$','$s_2(a)$'},'FontSize',20,'interpreter','latex','Location','NorthEast')
hold on
plot(linspace(amin1,amax1,I),zeros(1,I),'k--')
set(gca,'FontSize',16)
set(gca, 'XTick', []);
set(gca, 'YTick', []);
line([amin amin], [-.1 .08],'Color','Black','LineStyle','--')
line([0 0], [-.1 .08],'Color','Black','LineStyle','--')
xlabel('Wealth, $a$','interpreter','latex','FontSize',20)
ylabel('Saving, $s_j(a)$','interpreter','latex','FontSize',20)
xlim([amin1 amax1])
ylim([-0.04 0.08])
set(findobj('color','g'),'Color',[0 0.5 0]);
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure17a.eps

figure(2)
hold off
h1 = plot(a,g(:,1),'g--',a,g(:,2),'b',a,g(:,3),'r-.','LineWidth',2)
legend(h1,{'$g_0(a)$','$g_1(a)$','$g_2(a)$'},'FontSize',20,'interpreter','latex')
set(gca, 'XTick', []);
set(gca, 'YTick', []);
line([amin amin], [0 max(max(g(:,:)))],'Color','Black','LineStyle','--')
line([0 0], [0 max(max(g(:,:)))],'Color','Black','LineStyle','--')
xlabel('Wealth, $a$','interpreter','latex','FontSize',20)
ylabel('Densities, $g_j(a)$','interpreter','latex','FontSize',20)
xlim([amin1 amax1])
ylim([0 4])
set(findobj('color','g'),'Color',[0 0.5 0]);
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure17b.eps

