clearvars -except timerS own_machine_performance; clc; close all;


tic;

s = 2;
rho = 0.05;
z1 = .01;
z2 = 3*z1;
z = [z1,z2];
la1 = 0.01;
la2 = 0.01;
la = [la1,la2];

w = 3;
R1 = 0.051;
R2 = 0.051;
R = [R1,R2];
r = 0.041;
phi = 0.3;

zeta = 1.5;
sig2 = (zeta/s + 1)/2*(R2-r)^2/(rho - r); %pick sig2 so that zeta=1.5
sig = sig2^(1/2);

I= 5000;
amin = -phi;
amax = 1000;

%non-uniform grid
x = linspace(0,1,I)';
coeff = 5; power = 10;
xx  = x + coeff*x.^power;
xmax = max(xx); xmin = min(xx);
a = (amax-amin)/(xmax - xmin)*xx + amin;
daf = ones(I,1);
dab = ones(I,1);
daf(1:I-1) = a(2:I)-a(1:I-1);
dab(2:I) = a(2:I)-a(1:I-1);
daf(I)=daf(I-1); dab(1)=dab(2);

aa = [a,a];
daaf = daf*ones(1,2);
daab = dab*ones(1,2);

%objects for approximation of second derivatives
denom = 0.5*(daaf + daab).*daab.*daaf;
weightf = daab./denom;
weight0 = -(daab + daaf)./denom;
weightb = daaf./denom;


zz = ones(I,1)*z;
RR = ones(I,1)*R;
kk = ones(I,2);

maxit= 100;
crit = 10^(-6);
Delta = 1000;

dVf = zeros(I,2);
dVb = zeros(I,2);
dV0 = zeros(I,2);
dV2 = zeros(I,2);
c = zeros(I,2);
c0 = zeros(I,2);

Aswitch = [-speye(I)*la(1),speye(I)*la(1);speye(I)*la(2),-speye(I)*la(2)];

%INITIAL GUESS
v0(:,1) = (w*z(1) + r*a).^(1-s)/(1-s)/rho;
v0(:,2) = (w*z(2) + r*a).^(1-s)/(1-s)/rho;


risky_share = (R-r)/(s*sig2);
cslope = (rho - (1-s)*r)/s - (1-s)/(2*s)*(R-r).^2/(s*sig2);
sbar = (r-rho)/s + (1+s)/(2*s)*(R-r).^2/(s*sig2);

v = v0;

for n=1:maxit
    disp(n)
    V = v; 
    V_n(:,:,n)=V;
    % forward difference
    dVf(1:I-1,:) = (V(2:I,:)-V(1:I-1,:))./(aa(2:I,:) - aa(1:I-1,:));
    dVf(I,:) = (w*z + r.*amax).^(-s).*(1 - 1./cslope.*(R-r).^2/(s*sig2)).^s; %will never be used
    % backward difference
    dVb(2:I,:) = (V(2:I,:)-V(1:I-1,:))./(aa(2:I,:) - aa(1:I-1,:));
    dVb(1,:) = (w*z + r.*amin).^(-s); %state constraint boundary condition

    %second derivative: approximation only differs at amax
    dV2b(2:I-1,:) = (daab(2:I-1,:).*V(3:I,:) - (daab(2:I-1,:)+daaf(2:I-1,:)).*V(2:I-1,:) + daaf(2:I-1,:).*V(1:I-2,:))./denom(2:I-1,:);
    dV2f(2:I-1,:) = (daab(2:I-1,:).*V(3:I,:) - (daab(2:I-1,:)+daaf(2:I-1,:)).*V(2:I-1,:) + daaf(2:I-1,:).*V(1:I-2,:))./denom(2:I-1,:);
    dV2b(I,:) = -s*dVb(I,:).^(1+1/s).*cslope;
    dV2f(I,:) = -s*dVf(I,:).^(1+1/s).*cslope;
   
    I_concave = dVb > dVf; %indicator whether value function is concave (problems arise if this is not the case)
    
    %consumption and savings with forward difference
    cf = dVf.^(-1/s);
    kf = max(- dVf./dV2f.*(RR-r)/sig2,0);
    kf = min(kf,aa+phi);
    ssf = w*zz + (RR-r).*kf + r.*aa - cf;
    %consumption and savings with backward difference
    cb = dVb.^(-1/s);
    kb = max(- dVb./dV2b.*(RR-r)/sig2,0);
    kb = min(kb,aa+phi);
    ssb = w*zz + (RR-r).*kb + r.*aa - cb;
    %consumption and derivative of value function at steady state
    k0 = (kb + kf)/2; %could do something fancier here but the following seems to work well. And more importantly, it's fast
    c0 = w*zz + (RR-r).*k0 + r.*aa;
    dV0 = c0.^(-s);
   
    % dV_upwind makes a choice of forward or backward differences based on the sign of the drift    
    If = ssf > 10^(-12); %positive drift --> forward difference
    Ib = ssb < -10^(-12); %negative drift --> backward difference
    I0 = (1-If-Ib); %at steady state
   
    dV_Upwind = dVf.*If + dVb.*Ib + dV0.*I0; %important to include third term
    c = dV_Upwind.^(-1/s);
    u = c.^(1-s)/(1-s);
    k = max(-dV_Upwind./dV2b.*(RR-r)/sig2,0);
    k = min(k,aa+phi);
    
   
    %CONSTRUCT MATRIX
    X = -min(ssb,0)./daab + sig2/2.*k.^2.*weightb;
    Y = -max(ssf,0)./daaf + min(ssb,0)./daab + sig2/2.*k.^2.*weight0;
    Z = max(ssf,0)./daaf + sig2/2.*k.^2.*weightf;    
    
    xi = -dVb(I,:).^(-1/s)/cslope.*(R-r).^2/(2*s*sig2);
    X(I,:) = -min(ssb(I,:),0)./daab(I,:) - xi./daab(I,:);
    Y(I,:) = -max(ssf(I,:),0)./daaf(I,:) + min(ssb(I,:),0)./daab(I,:) + xi./daab(I,:);
    Z(I,:) = max(ssf(I,:),0)./daaf(I,:);
    
    A1=spdiags(Y(:,1),0,I,I)+spdiags(X(2:I,1),-1,I,I)+spdiags([0;Z(1:I-1,1)],1,I,I);
    A2=spdiags(Y(:,2),0,I,I)+spdiags(X(2:I,2),-1,I,I)+spdiags([0;Z(1:I-1,2)],1,I,I);
    A1(I,I) = Y(I,1) + Z(I,1);
    A2(I,I) = Y(I,2) + Z(I,2);
    A = [A1,sparse(I,I);sparse(I,I),A2] + Aswitch;
    
    B = (1/Delta + rho)*speye(2*I) - A;
    
    u_stacked = [u(:,1);u(:,2)];
    V_stacked = [V(:,1);V(:,2)];
    
    b = u_stacked + V_stacked/Delta;
    V_stacked = B\b; %SOLVE SYSTEM OF EQUATIONS
    
    V = [V_stacked(1:I),V_stacked(I+1:2*I)];
    
    Vchange = V - v;
    v = V;

    dist(n) = max(max(abs(Vchange)));
    if dist(n)<crit
        disp('Value Function Converged, Iteration = ')
        disp(n)
        break
    end
end
toc;

%%%%%%%%%%%%%%%%%%%%%%%%%%
% FOKKER-PLANCK EQUATION %
%%%%%%%%%%%%%%%%%%%%%%%%%%

%RECOMPUTE TRANSITION MATRIX WITH REFLECTING BARRIER AT amax
X = -min(ssb,0)./daab + sig2/2.*k.^2.*weightb;
Y = -max(ssf,0)./daaf + min(ssb,0)./daab + sig2/2.*k.^2.*weight0;
Z = max(ssf,0)./daaf + sig2/2.*k.^2.*weightf; 

A1=spdiags(Y(:,1),0,I,I)+spdiags(X(2:I,1),-1,I,I)+spdiags([0;Z(1:I-1,1)],1,I,I);
A2=spdiags(Y(:,2),0,I,I)+spdiags(X(2:I,2),-1,I,I)+spdiags([0;Z(1:I-1,2)],1,I,I);
A1(I,I) = Y(I,1) + Z(I,1);
A2(I,I) = Y(I,2) + Z(I,2);
A = [A1,sparse(I,I);sparse(I,I),A2] + Aswitch;

%ADJUST TO ENSURE MASS-PRESERVATION WITH NON-UNIFORM GRID
da_tilde = 0.5*(dab + daf);
da_tilde(1) = 0.5*daf(1); da_tilde(I) = 0.5*dab(I);
da_stacked = [da_tilde;da_tilde];
grid_diag = spdiags(da_stacked,0,2*I,2*I);
A = grid_diag*A*grid_diag^(-1);

AT = A';
b = zeros(2*I,1);

%need to fix one value, otherwise matrix is singular
i_fix = 1;
b(i_fix)=.1;
row = [zeros(1,i_fix-1),1,zeros(1,2*I-i_fix)];
AT(i_fix,:) = row;

%Solve linear system
gg = AT\b;

g_sum = gg'*da_stacked;
gg = gg./g_sum;

g = [gg(1:I),gg(I+1:2*I)];

check1 = g(:,1)'*da_tilde;
check2 = g(:,2)'*da_tilde;

%CALCULATE THEORETICAL POWER LAW EXPONENT
zeta = s*(2*sig2*(rho - r)/(R2-r)^2 -1);

adot = w*zz + (RR-r).*k + r.*aa - c;
plot(a,k,a,(ones(I,1)*risky_share).*aa)
plot(a,c,a,(ones(I,1)*cslope).*aa)

%COMPUTE DISTRIBUTION OF x=log(a)
for i=1:I
    G(i,1) = sum(g(1:i,1).*da_tilde(1:i));
    G(i,2) = sum(g(1:i,2).*da_tilde(1:i));
end

f = zeros(I,2);
x = log(max(a,0));
dx = zeros(I,1);
for i =2:I
dx(i) = x(i)-x(i-1);
f(i,1) = (G(i,1)-G(i-1,1))/dx(i);
f(i,2) = (G(i,2)-G(i-1,2))/dx(i);
end
f(1)=0;

xmin = log(1); xmax = log(amax);
ga = g(1:i,1) + g(1:i,2);
fx = f(:,1)+f(:,2);

%Plot sizes
FigureSize = [19.5 15.5];
lineWidth = 2.2;
lineWidth2 = 0.5;

amax1 = 40;
amin1 = amin-2;
figure(1)
h1 = plot(a,adot(:,1),'b',a,adot(:,2),'r','LineWidth',lineWidth)
hold on 
line(linspace(amin,amax,I),zeros(1,I),...
    'Color','Black',...
    'LineStyle','--','LineWidth',lineWidth2)
legend(h1,'s_1(a)','s_2(a)','Location','NorthEast')
set(gca,'FontSize',16)
text(-0.555,-.085,'$\underline{a}$','FontSize',16,'interpreter','latex')
line([amin amin], [-.08 .06],'Color','Black',...
    'LineStyle','--','LineWidth',lineWidth2)
xlabel('Wealth, $a$','interpreter','latex')
ylabel('Savings, $s_i(a)$','interpreter','latex')
xlim([amin1 amax1])
set(gca, 'XTick', []);
set(gca, 'YTick', []);
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure18a.eps

amax1 = 10;
amin1 = amin-0.3;
figure(2)
h1 = plot(a,k(:,1),'b',a,k(:,2),'r','LineWidth',lineWidth)
legend(h1,'k_1(a)','k_2(a)','Location','NorthWest')
set(gca,'FontSize',16)
text(-0.2,-.2,'$\underline{a}$','FontSize',16,'interpreter','latex')
line([amin amin], [0 7],'Color','Black','LineStyle','--')
xlabel('Wealth, $a$','interpreter','latex')
ylabel('Risky Assets, $k_i(a)$','interpreter','latex')
xlim([amin1 amax1])
set(gca, 'XTick', []);
set(gca, 'YTick', []);
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure18b.eps

figure(3)
h1 = plot(a,g(:,1),'b',a,g(:,2),'r','LineWidth',lineWidth)
legend(h1,'g_1(a)','g_2(a)')
set(gca,'FontSize',16)
text(-0.555,-.015,'$\underline{a}$','FontSize',16,'interpreter','latex')
line([amin amin], [0 max(max(g))],'Color','Black','LineStyle','--')
xlabel('Wealth, $a$','interpreter','latex')
ylabel('Densities, $g_i(a)$','interpreter','latex')
xlim([amin1 amax1])
ylim([0 0.3])
set(gca, 'XTick', []);
set(gca, 'YTick', []);
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure18c.eps

figure(4)
h1 = plot(x,log(f(:,1)),'b',x,log(f(:,2)),'r',x,-zeta*x','k--' ...
    ,'LineWidth',lineWidth)
legend(h1,'log(f_1(x))','log(f_2(x))','Slope = -\zeta')
set(gca,'FontSize',16)
xlabel('Log Wealth, $x=\log \ (a)$','interpreter','latex')
ylabel('$\log \ (f_i(x))$','interpreter','latex')
%xlim([1 log(13.5)])
ylim([-10 0])
set(gca, 'XTick', []);
set(gca, 'YTick', []);
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure18d.eps




