clearvars -except timerS own_machine_performance; clc; close all;

tic;

s = 2;
rho = 0.05;
z1 = .1;
z2 = .2;
z = [z1,z2];
la1 = 0.001;
la2 = 0.001;
la1 = 1.5;
la2 = 1.6;
la = [la1,la2];

Aprod = 0.8;
al = 1/3;
d = 0.05;

I=1000;
amin = -0.15;
amax = 4;
a = linspace(amin,amax,I)';
da = (amax-amin)/(I-1);

aa = [a,a];
zz = ones(I,1)*z;


maxit= 100;
crit = 10^(-6);
Delta = 40;

dVf = zeros(I,2);
dVb = zeros(I,2);
c = zeros(I,2);

Aswitch = [eye(I)*la(1),-eye(I)*la(1);-eye(I)*la(2),eye(I)*la(2)];

Ir = 2;
% rmin = 0.00001;
rmin = 0.025;
rmax = 0.04;
r_grid = linspace(rmin,rmax,Ir);

%INITIAL GUESS
r = r_grid(1);
KD = (al*Aprod/(r+d))^(1/(1-al));
w = (1-al)*Aprod*KD^al;
% v0(:,1) = (z(1) + r.*a).^(1-s)/(1-s)/rho;
% v0(:,2) = (z(2) + r.*a).^(1-s)/(1-s)/rho;
v0(:,1) = (w*z(1) + max(r,0.01).*a).^(1-s)/(1-s)/rho;
v0(:,2) = (w*z(2) + max(r,0.01).*a).^(1-s)/(1-s)/rho;

for ir=1:Ir;

r = r_grid(ir);
KD(ir) = (al*Aprod/(r+d))^(1/(1-al));
w = (1-al)*Aprod*KD(ir)^al;

if ir>1
v0 = V_r(:,:,ir-1);
end

v = v0;

for n=1:maxit
    V = v;
    V_n(:,:,n)=V;
    % forward difference
    dVf(1:I-1,:) = (V(2:I,:)-V(1:I-1,:))/da;
    dVf(I,:) = (w*z + r.*amax).^(-s); %will never be used
    % backward difference
    dVb(2:I,:) = (V(2:I,:)-V(1:I-1,:))/da;
    dVb(1,:) = (w*z + r.*amin).^(-s); %state constraint boundary condition
    
    I_concave = dVb > dVf; %indicator whether value function is concave (problems arise if this is not the case)
    
    %consumption and savings with forward difference
    cf = max(dVf,10^(-10)).^(-1/s);
    ssf = w*zz + r.*aa - cf;
    %consumption and savings with backward difference
    cb = max(dVb,10^(-10)).^(-1/s);
    ssb = w*zz + r.*aa - cb;
    %consumption and derivative of value function at steady state
    c0 = w*zz + r.*aa;
        
    % upwind scheme makes a choice of forward or backward differences based on
    % the sign of the drift    
    If = ssf > 0; %positive drift --> forward difference
    Ib = ssb < 0; %negative drift --> backward difference
    I0 = (1-If-Ib); %at steady state
    %make sure backward difference is used at amax
    
    c = cf.*If + cb.*Ib + c0.*I0;
    u = c.^(1-s)/(1-s);
    
    %CONSTRUCT MATRIX
    X = min(ssb,0)/da;
    Y = max(ssf,0)/da - min(ssb,0)/da;
    Z = -max(ssf,0)/da;
    
    A1=spdiags(Y(:,1),0,I,I)+spdiags(X(2:I,1),-1,I,I)+spdiags([0;Z(1:I-1,1)],1,I,I);
    A2=spdiags(Y(:,2),0,I,I)+spdiags(X(2:I,2),-1,I,I)+spdiags([0;Z(1:I-1,2)],1,I,I);
    A = [A1,sparse(I,I);sparse(I,I),A2] + Aswitch;
    
    B = A + (1/Delta + rho)*speye(2*I);

    u_stacked = [u(:,1);u(:,2)];
    V_stacked = [V(:,1);V(:,2)];
    
    b = u_stacked + V_stacked/Delta;
    V_stacked = B\b; %SOLVE SYSTEM OF EQUATIONS
    
    V = [V_stacked(1:I),V_stacked(I+1:2*I)];
    
    Vchange = V - v;
    v = V;

    dist(n) = max(max(abs(Vchange)));
    if dist(n)<crit
        disp('Value Function Converged, Iteration = ')
        disp(n)
        break
    end
end

toc;

%%%%%%%%%%%%%%%%%%%%%%%%%%
% FOKKER-PLANCK EQUATION %
%%%%%%%%%%%%%%%%%%%%%%%%%%
AT = A';
b = zeros(2*I,1);

%need to fix one value, otherwise matrix is singular
i_fix = 1;
b(i_fix)=.1;
row = [zeros(1,i_fix-1),1,zeros(1,2*I-i_fix)];
AT(i_fix,:) = row;

%Solve linear system
gg = AT\b;
g_sum = gg'*ones(2*I,1)*da;
gg = gg./g_sum;

g = [gg(1:I),gg(I+1:2*I)];

check1 = g(:,1)'*ones(I,1)*da;
check2 = g(:,2)'*ones(I,1)*da;

g_r(:,:,ir) = g;
adot(:,:,ir) = w*zz + r.*aa - c;
V_r(:,:,ir) = V;
c_r(:,:,ir) = c;

S(ir) = g(:,1)'*a*da + g(:,2)'*a*da;
end


%Plot sizes
FigureSize = [20 16];

ir = 1;
amax1 = 0.4;
amin1 = amin-0.03;
cmax = 0.4;
h1 = plot(a,c_r(:,1,ir),'b',a,c_r(:,2,ir),'r-.','LineWidth',2)
set(gca,'FontSize',16)
legend(h1,{'$c_1(a)$','$c_2(a)$'},'FontSize',20 ...
    ,'interpreter','latex','Location','NorthWest' ...
    ,'AutoUpdate','off')
set(gca, 'XTick', []);
set(gca, 'YTick', []);
text(-0.155,0.0865,'$\underline{a}$','FontSize',20,'interpreter','latex')
line([amin amin], [0.09 .2],'Color','Black','LineStyle','--')
xlabel('Wealth, $a$','interpreter','latex','FontSize',20)
ylabel('Consumption, $c_j(a)$','interpreter','latex','FontSize',20)
xlim([amin1 amax1])
ylim([0.09 .2])
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure1a.eps


hold off

ir = 1;
amax1 = 0.4;
amin1 = amin-0.03;
h1 = plot(a,adot(:,1,ir),'b',a,adot(:,2,ir),'r-.','LineWidth',2)
legend(h1,{'$s_1(a)$','$s_2(a)$'},'FontSize',20,'interpreter','latex','Location','NorthEast')
hold on
plot(linspace(amin1,amax1,I),zeros(1,I),'k--')
set(gca,'FontSize',16)
set(gca, 'XTick', []);
set(gca, 'YTick', []);
text(-0.155,-.105,'$\underline{a}$','FontSize',20,'interpreter','latex')
line([amin amin], [-.1 .08],'Color','Black','LineStyle','--')
xlabel('Wealth, $a$','interpreter','latex','FontSize',20)
ylabel('Saving, $s_j(a)$','interpreter','latex','FontSize',20)
xlim([amin1 amax1])
ylim([-.1 .08])
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure1b.eps
print -depsc ../Figures/Figure6a.eps

hold off

h1 = plot(a,g_r(:,1,ir),'b',a,g_r(:,2,ir),'r-.','LineWidth',2)
set(gca,'FontSize',16)
legend(h1,{'$g_1(a)$','$g_2(a)$'},'FontSize',20,'interpreter','latex')
set(gca, 'XTick', []);
set(gca, 'YTick', []);
text(-0.155,-.12,'$\underline{a}$','FontSize',20,'interpreter','latex')
line([amin amin], [0 max(max(g_r(:,:,ir)))],'Color','Black','LineStyle','--')
xlabel('Wealth, $a$','interpreter','latex','FontSize',20)
ylabel('Densities, $g_j(a)$','interpreter','latex','FontSize',20)
xlim([amin1 amax1])
ylim([0 4.5])
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure6b.eps

%%%%%%%%%%%%%%%%%
% INCREASE IN r %
%%%%%%%%%%%%%%%%%
hold off

amax1 = 0.4;
amin1 = amin-0.03;
h1 = plot(a,adot(:,1,1),'b--',a,adot(:,2,1),'r--',a,adot(:,1,2),'b',a,adot(:,2,2),'r','LineWidth',2)
set(gca,'FontSize',16)
%legend(h1,{'$s_1(a;r_L)$','$s_2(a;r_L)$','$s_1(a;r_H)$','$s_2(a;r_H)$'},'FontSize',20,'interpreter','latex','Location','East')

hold on

plot(linspace(amin1,amax1,I),zeros(1,I),'k--')
set(gca, 'XTick', []);
set(gca, 'YTick', []);
text(0.2,-0.085,'$s_1(a;r_L)$','FontSize',20,'interpreter','latex')
text(0.25,-.048,'$s_1(a;r_H)$','FontSize',20,'interpreter','latex')
text(0.25,0.011,'$s_2(a;r_L)$','FontSize',20,'interpreter','latex')
text(0.29,0.041,'$s_2(a;r_H)$','FontSize',20,'interpreter','latex')
text(-0.155,-.105,'$\underline{a}$','FontSize',20,'interpreter','latex')
line([amin amin], [-.1 .08],'Color','Black','LineStyle','--')
xlabel('Wealth, $a$','interpreter','latex','FontSize',20)
ylabel('Saving, $s_j(a)$','interpreter','latex','FontSize',20)
xlim([amin1 amax1])
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure7a.eps

hold off

g_r(1,1,2)=g_r(1,1,1);
h1 = plot(a,g_r(:,1,1),'b--',a,g_r(:,2,1),'r--',a,g_r(:,1,2),'b',a,g_r(:,2,2),'r','LineWidth',2)
set(gca,'FontSize',16)
legend(h1,{'$g_1(a;r_L)$','$g_2(a;r_L)$','$g_1(a;r_H)$','$g_2(a;r_H)$'},'FontSize',20,'interpreter','latex')
set(gca, 'XTick', []);
set(gca, 'YTick', []);
text(-0.155,-.12,'$\underline{a}$','FontSize',20,'interpreter','latex')
line([amin amin], [0 max(max(g_r(:,:,ir)))],'Color','Black','LineStyle','--')
xlabel('Wealth, $a$','interpreter','latex','FontSize',20)
ylabel('Densities, $g_j(a)$','interpreter','latex','FontSize',20)
xlim([amin1 amax1])
ylim([0 4.5])
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure7b.eps

