clearvars -except timerS own_machine_performance; clc; close all;

the = 40;
ga = 0.5;
r = 0.02;
rho = 0.05;
nu = (rho-r)/the;
y=1;
a0 = 1;

N=100;
t = linspace(0,100,N);

T = sqrt(2*a0/nu);
c_exp = y + nu*max(T-t,0);
adot_exp = -nu*max(T-t,0);
a_exp = nu/2*(max(T-t,0)).^2;

eta = (rho-r)/ga;
a_CRRA = a0*exp(-eta*t);
c_CRRA = (r+nu)*a_CRRA;
adot_CRRA = -nu*a_CRRA;



%define plot sizes 
fontSize = 8;
lineWidth = 1;
lineWidth2 = 0.5;
FigureSize = [20 5.5];

figure(1)
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)

subplot(1,3,1)
plot(t,c_exp,'b','LineWidth',lineWidth)
hold on
plot(t,y*ones(N,1),'k--','LineWidth', lineWidth2)
set(gca,'FontSize',fontSize)
set(gca, 'XTick', []);
set(gca, 'YTick', []);
ylim([0.999 max(c_exp)])
text(-6,1,'$y$','FontSize',fontSize,'interpreter','latex')
xlabel('Time, $t$','interpreter','latex')
ylabel('Consumption, $c(t)$','interpreter','latex')
title('(a) Consumption','interpreter','latex')


subplot(1,3,2)
plot(t,adot_exp,'b','LineWidth',lineWidth)
hold on 
plot(t,zeros(N,1),'k--','LineWidth',lineWidth2)
set(gca,'FontSize',fontSize)
set(gca, 'XTick', []);
set(gca, 'YTick', []);
ylim([min(adot_exp) 0.001])
text(-6,0,'$0$','FontSize',fontSize,'interpreter','latex')
xlabel('Time, $t$','interpreter','latex')
ylabel('Savings, $\dot{a}(t)$','interpreter','latex')
title('(b) Savings','interpreter','latex')

subplot(1,3,3)
plot(t,a_exp,'b','LineWidth',lineWidth)
hold on
plot(t,zeros(N,1),'k--','LineWidth',lineWidth2)
set(gca,'FontSize',fontSize)
set(gca, 'XTick', []);
set(gca, 'YTick', []);
ylim([-0.05 a0])
text(-6,0,'$0$','FontSize',fontSize,'interpreter','latex')
xlabel('Time, $t$','interpreter','latex')
ylabel('Wealth, $a(t)$','interpreter','latex')
title('(c) Wealth','interpreter','latex')

print -depsc ../Figures/Figure2.eps

figure(2)
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)

subplot(1,3,1)
plot(t,c_CRRA,'b','LineWidth',lineWidth)
hold on
plot(t,zeros(N,1),'k--','LineWidth',lineWidth2)
set(gca,'FontSize',fontSize)
set(gca, 'XTick', []);
set(gca, 'YTick', []);
ylim([-.001 max(c_CRRA)])
text(-6,0,'$0$','FontSize',fontSize,'interpreter','latex')
xlabel('Time, $t$','interpreter','latex')
ylabel('Consumption, $c(t)$','interpreter','latex')
title('(a) Consumption','interpreter','latex')


subplot(1,3,2)
plot(t,adot_CRRA,'b','LineWidth',lineWidth)
hold on 
plot(t,zeros(N,1),'k--','LineWidth',lineWidth2)
set(gca,'FontSize',fontSize)
set(gca, 'XTick', []);
set(gca, 'YTick', []);
ylim([min(adot_CRRA) 0.00001])
text(-6,0,'$0$','FontSize',fontSize,'interpreter','latex')
xlabel('Time, $t$','interpreter','latex')
ylabel('Savings, $\dot{a}(t)$','interpreter','latex')
title('(b) Savings','interpreter','latex')

subplot(1,3,3)
plot(t,a_CRRA,'b','LineWidth',lineWidth)
hold on 
plot(t,zeros(N,1),'k--','LineWidth',lineWidth2)
set(gca,'FontSize',fontSize)
set(gca, 'XTick', []);
set(gca, 'YTick', []);
ylim([-0.05 a0])
text(-6,0,'$0$','FontSize',fontSize,'interpreter','latex')
xlabel('Time, $t$','interpreter','latex')
ylabel('Wealth, $a(t)$','interpreter','latex')
title('(c) Wealth','interpreter','latex')

print -depsc ../Figures/Figure3.eps

