clearvars -except timerS own_machine_performance; clc; close all;

s = 2.5;
r = 0.035;
rho = 0.05;
z1 = .1;
z2 = .2;
la1 = 0.4;
la2 = 0.2;
tau = 2;

eta = (rho-r)/s;
MPC_cf = tau*(eta+r);

I=500;
amin = 0;
amax = 3;
a = linspace(amin,amax,I)';


N = 25;
z1_grid = linspace(0.005,z2,N);
for i=1:N
    z1=z1_grid(i);
    params = [s,r,rho,z1,z2,la1,la2,tau];
    grid_params = [amin,amax,I];
    [M1(i),nu(i),s1der(i),c1(:,i),c2(:,i),MPC1(:,i),MPC2(:,i),MPC1_PV(:,i)] = partialeq_subroutine(params,grid_params);
    p1(i) = la2/(la1+la2);
    M1(i)=M1(i)/p1(i);
end

% For rich people, using numerical MPC (e.g. MPC1(I-30,:)) introduces slight numerical error due to too tight a state space (people become close to amax).
% Get rid of this by using closed form MPC_cf = tau*(eta+r).
% But plot looks very similar if replace plot(z1_grid,MPC_cf*ones(N,1)) by plot(z1_grid,MPC1(I-30,:))

%Plot sizes
FigureSize = [20 16];

h1 = plot(z1_grid,MPC1(1,:),'r-.',z1_grid,MPC1(5,:),'b',z1_grid,MPC_cf*ones(N,1),'g--','LineWidth',2)
set(gca,'FontSize',16)
legend({'Lowest Wealth','50th Percentile Wealth','95th Percentile Wealth'},'interpreter','latex','FontSize',20,'Location','SouthEast')
xlabel('Low Income Realization $y_1$','interpreter','latex','FontSize',20)
ylabel('$\textrm{MPC}_{1,\tau}(a)$','interpreter','latex','FontSize',20)
xlim([min(z1_grid) z2])
ylim([0 1])
set(findobj('color','g'),'Color',[0 0.5 0]);
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure5a.eps


clear MPC1 MPC2 c1 c2

z1 = 0.1; amin = 0;

N = 40;
z2_grid = linspace(z1,0.3,N);
for i=1:N
    z2=z2_grid(i);
    params = [s,r,rho,z1,z2,la1,la2,tau];
    grid_params = [amin,amax,I];
    [M1(i),nu(i),s1der(i),c1(:,i),c2(:,i),MPC1(:,i),MPC2(:,i),MPC1_PV(:,i)] = partialeq_subroutine(params,grid_params);
    p1(i) = la2/(la1+la2);
    M1(i)=M1(i)/p1(i);
end

h1 = plot(z2_grid,MPC1(1,:),'r-.',z2_grid,MPC1(5,:),'b',z2_grid,MPC_cf*ones(N,1),'g--','LineWidth',2)
set(gca,'FontSize',16)
legend({'Lowest Wealth','50th Percentile Wealth','95th Percentile Wealth'},'interpreter','latex','FontSize',20,'Location','SouthEast')
xlabel('High Income Realization $y_2$','interpreter','latex','FontSize',20)
ylabel('$\textrm{MPC}_{1,\tau}(a)$','interpreter','latex','FontSize',20)
xlim([min(z2_grid) max(z2_grid)])
ylim([0 1])
set(findobj('color','g'),'Color',[0 0.5 0]);
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure5b.eps



