clearvars -except timerS own_machine_performance; clc; close all;

%% Run simulations in auxiliary files

%use your machine's run instead
if  own_machine_performance
    run('disc_true_dist.m')  %finer grid discrete distribution
    run('cont_true_dist.m')  %finer grid continuous distribution
    run('main_dist.m')       %coarser grid distributions 
end

%% Read plot data from file

%file with simulation results
fname= '../Data/plot_data_manuscript.txt'; %run from manuscript
if own_machine_performance
   fname =  '../Data/plot_data.txt';       %your own run
end
pdata = csvread(fname);

grid_size = pdata(1,:)';
accuracy_gain = pdata(2,:)';
error_cont = pdata(3,:)';
error_disc = pdata(4,:)';
error_aggc_cont = pdata(5,:)';
error_aggc_disc = pdata(6,:)';
speed_cont = pdata(7,:)';
speed_disc = pdata(8,:)';
agg_speed_cont = pdata(10,:)';
agg_speed_disc = pdata(11,:)';


%% Plots
%Plot sizes
FigureSize = [19.7 15];

speed_c_adpt = interp1(error_cont, speed_cont, error_disc, 'pch');
speed_gain = speed_disc./speed_c_adpt;

agg_speed_d_adpt = interp1(error_aggc_disc, agg_speed_disc, error_aggc_cont, 'pch');
agg_speed_gain = agg_speed_d_adpt./agg_speed_cont;


semilogx(error_cont, speed_cont, 'b-o', error_disc, speed_disc, 'r-x', 'LineWidth', 1.5);
set(gca,'FontSize',14)
grid on;
xlabel('Policy function error in \%','interpreter','latex','FontSize',20);
ylabel('Speed in seconds','interpreter','latex','FontSize',20);
legend('Continuous time (finite difference method)','Discrete time (endogenous grid method)','interpreter','latex','FontSize',18);
ylim([-1,20]);
xlim([min(error_cont)-0.0002 1])
xticklabels({'0.01','0.1','1'})
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure9a.eps


semilogx(error_aggc_cont, agg_speed_cont, 'b-o', error_aggc_disc, agg_speed_disc, 'r-x', 'LineWidth', 1.5);
set(gca,'FontSize',14)
grid on;
xlabel('Error in aggregate consumption in \%','interpreter','latex','FontSize',20);
ylabel('Speed in seconds','interpreter','latex','FontSize',20);
legend('Continuous time (finite difference method)','Discrete time (endogenous grid method)','interpreter','latex','FontSize',18);
%ylim([-1,16]);
xlim([min(error_aggc_cont)-0.0002 0.2])
xticklabels({'0.01','0.1','1'})
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure9b.eps

semilogx(error_disc, speed_gain, 'r-x', 'LineWidth', 1.5);
set(gca,'FontSize',14)
grid on;
xlabel('Policy function error in \%','interpreter','latex','FontSize',20);
ylabel('Speed gain of cont. vs discrete time','interpreter','latex','FontSize',20);
ylim([-1,30]);
xlim([min(error_cont)-0.0002 1])
xticklabels({'0.01','0.1','1'})
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure11a.eps

semilogx(error_aggc_cont, agg_speed_gain, 'r-x', 'LineWidth', 1.5);
set(gca,'FontSize',14)
grid on;
xlabel('Error in aggregate consumption in \%','interpreter','latex','FontSize',20);
ylabel('Speed gain of cont. vs discrete time','interpreter','latex','FontSize',20);
ylim([-1,500]);
xlim([min(error_aggc_cont)-0.0002 0.2])
xticklabels({'0.01','0.1','1'})
set(gcf,'Position', [3 3, FigureSize])
set(gcf,'PaperSize', FigureSize)
print -depsc ../Figures/Figure11b.eps
