function latex = latexTable2(input_p, input_a)
% A function to automatically generate the CT vs. DT comparison table
% in the HACT paper. Based on a code by Eli Duenisch
% ----------------------------------------------------------------------------------


% Default mode for applying input.tableDataFormat:
if ~isfield(input_p,'dataFormatMode'),input_p.dataFormatMode = 'column';end
% Sets the default display format of numeric values in the LaTeX table to '%.4f'
% (4 digits floating point precision).
if ~isfield(input_p,'dataFormat'),input_p.dataFormat = {'%.4f'};end

% Specify the alignment of the columns:
% 'l' for left-justified, 'c' for centered, 'r' for right-justified
if ~isfield(input_p,'tableColumnAlignment'),input_p.tableColumnAlignment = 'c';end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% process table datatype
if isa(input_p.data,'table')
  if(~isempty(input_p.data.Properties.RowNames))
    input_p.tableRowLabels = input_p.data.Properties.RowNames';
  end
  if(~isempty(input_p.data.Properties.VariableNames))
    input_p.tableColLabels = input_p.data.Properties.VariableNames';
  end
    input_p.data = table2array(input_p.data);
end
% get size of data
numberDataRows = size(input_p.data,1);
numberDataCols = size(input_p.data,2);
% obtain cell array for the table data and labels
colLabelsExist = isfield(input_p,'tableColLabels');
rowLabelsExist = isfield(input_p,'tableRowLabels');
cellSize = [numberDataRows+colLabelsExist,numberDataCols+rowLabelsExist];
C = cell(cellSize);
C(1+colLabelsExist:end,1+rowLabelsExist:end) = num2cell(input_p.data);
if rowLabelsExist
    C(1+colLabelsExist:end,1)=input_p.tableRowLabels';
end
if colLabelsExist
    C(1,1+rowLabelsExist:end)=input_p.tableColLabels;
end
% obtain cell array for the format
lengthDataFormat = length(input_p.dataFormat);
if lengthDataFormat==1
    tmp = repmat(input_p.dataFormat(1),numberDataRows,numberDataCols);
else
    dataFormatList={};
    for i=1:2:lengthDataFormat
        dataFormatList(end+1:end+input_p.dataFormat{i+1},1) = repmat(input_p.dataFormat(i),input_p.dataFormat{i+1},1);
    end
    if strcmp(input_p.dataFormatMode,'column')
        tmp = repmat(dataFormatList',numberDataRows,1);
    end
    if strcmp(input_p.dataFormatMode,'row')
        tmp = repmat(dataFormatList,1,numberDataCols);
    end
end

dataFormatArray = cell(cellSize);
dataFormatArray(1+colLabelsExist:end,1+rowLabelsExist:end) = tmp;

%% make table header lines (and policy header lines):
hLine = '\hline';

latex = {'\begin{center}';...
    '\begin{tabular}{c|cc|cc}';...
    '\multicolumn{5}{c}{(a) Policy Function} \\';...
    hLine;...
    '& \multicolumn{2}{c|}{Continuous time} & \multicolumn{2}{c}{Discrete time}\\';...
    '& \multicolumn{2}{c|}{(finite difference method)} & \multicolumn{2}{c}{(endogenous grid method)}\\';...
    hLine;...
    'Grid & Speed in & Policy function   &   Speed in  & Policy function   \\';...
    'points  & seconds  &  error (\%) &  seconds & error (\%) \\';...
    '(1) & (2) & (3)&  (4) & (5) \\'};
    
%% generate the policy table data
 
latex(end+1) = {hLine};
nf = java.text.DecimalFormat;
for i=1:size(C,1)
    
    rowStr = '';
    for j=1:size(C,2)
        dataValue = C{i,j};
        if iscell(dataValue)
          dataValue = dataValue{:};
        elseif isnumeric(dataValue) && floor(dataValue) ~= dataValue
          dataValue = num2str(dataValue,dataFormatArray{i,j});
        else
            dataValue = char(nf.format(dataValue));
        end
        if (i==size(C,1) && j==size(C,2))
            dataValue = '-';
        end
        if j==1
            rowStr = dataValue;
        else
            rowStr = [rowStr,' & ',dataValue];
        end
        
    end
    latex(end+1) = {[rowStr,' \\']};
end
latex(end+1) = {hLine};
%latex(end+1) = {hLine};


%% change the data table C to aggregate consumption data
% Default mode for applying input.tableDataFormat:
if ~isfield(input_a,'dataFormatMode'),input_a.dataFormatMode = 'column';end
% Sets the default display format of numeric values in the LaTeX table to '%.4f'
% (4 digits floating point precision).
if ~isfield(input_a,'dataFormat'),input_a.dataFormat = {'%.4f'};end

% Specify the alignment of the columns:
% 'l' for left-justified, 'c' for centered, 'r' for right-justified
if ~isfield(input_a,'tableColumnAlignment'),input_a.tableColumnAlignment = 'c';end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% process table datatype
if isa(input_a.data,'table')
  if(~isempty(input_a.data.Properties.RowNames))
    input_a.tableRowLabels = input_a.data.Properties.RowNames';
  end
  if(~isempty(input_a.data.Properties.VariableNames))
    input_a.tableColLabels = input_a.data.Properties.VariableNames';
  end
    input_a.data = table2array(input_a.data);
end
% get size of data
numberDataRows = size(input_a.data,1);
numberDataCols = size(input_a.data,2);
% obtain cell array for the table data and labels
colLabelsExist = isfield(input_a,'tableColLabels');
rowLabelsExist = isfield(input_a,'tableRowLabels');
cellSize = [numberDataRows+colLabelsExist,numberDataCols+rowLabelsExist];
C = cell(cellSize);
C(1+colLabelsExist:end,1+rowLabelsExist:end) = num2cell(input_a.data);
if rowLabelsExist
    C(1+colLabelsExist:end,1)=input_a.tableRowLabels';
end
if colLabelsExist
    C(1,1+rowLabelsExist:end)=input_a.tableColLabels;
end
% obtain cell array for the format
lengthDataFormat = length(input_a.dataFormat);
if lengthDataFormat==1
    tmp = repmat(input_a.dataFormat(1),numberDataRows,numberDataCols);
else
    dataFormatList={};
    for i=1:2:lengthDataFormat
        dataFormatList(end+1:end+input_a.dataFormat{i+1},1) = repmat(input_a.dataFormat(i),input_a.dataFormat{i+1},1);
    end
    if strcmp(input_a.dataFormatMode,'column')
        tmp = repmat(dataFormatList',numberDataRows,1);
    end
    if strcmp(input_a.dataFormatMode,'row')
        tmp = repmat(dataFormatList,1,numberDataCols);
    end
end

dataFormatArray = cell(cellSize);
dataFormatArray(1+colLabelsExist:end,1+rowLabelsExist:end) = tmp;

%% make aggregate consumption sub_table header lines:
hLine = '\hline';

latex_t = {'\multicolumn{5}{c}{(b) Aggregate Consumption} \\';...
    hLine;...
    '& \multicolumn{2}{c|}{Continuous time} & \multicolumn{2}{c}{Discrete time} \\';...
    '& \multicolumn{2}{c|}{(finite difference method)} & \multicolumn{2}{c}{(endogenous grid method)} \\';...
    hLine;...
    'Grid & Speed in & Error in aggregate  &   Speed in  & Error in aggregate \\';...
    'points  & seconds  &  consumption (\%) &  seconds &  consumption (\%) \\';...
    '(6) & (7) & (8)&  (9) & (10) \\'};
    
latex = [latex;latex_t];

%% generate the aggregate consumption table data
latex(end+1) = {hLine};
nf = java.text.DecimalFormat;
for i=1:size(C,1)
    
    rowStr = '';
    for j=1:size(C,2)
        dataValue = C{i,j};
        if iscell(dataValue)
          dataValue = dataValue{:};
        elseif isnumeric(dataValue) && floor(dataValue) ~= dataValue
          dataValue = num2str(dataValue,dataFormatArray{i,j});
        else
            dataValue = char(nf.format(dataValue));
        end
        if (i==size(C,1) && j==size(C,2))
            dataValue = '-';
        end
        if j==1
            rowStr = dataValue;
        else
            rowStr = [rowStr,' & ',dataValue];
        end
        
    end
    latex(end+1) = {[rowStr,' \\']};
end
latex(end+1) = {hLine};
latex(end+1) = {hLine};



%% make footer lines for table:
% tableFooter = {'\end{tabular}';['\caption{',input.tableCaption,'}']; ...
%     ['\label{table:',input.tableLabel,'}'];'\end{table}'};
tableFooter = {'\end{tabular}';'\end{center}'};

% if input.tableBorders
%     latex = [latex;{hLine};tableFooter];
% else
     latex = [latex;tableFooter];
% end
% print latex code to console:
disp(char(latex));
end
