clearvars -except timerS own_machine_performance; clc; close all;

%format long g;
%% Shared Variables
% preferences: CRRA utility
risk_aver   = 2;
beta        = 0.95;
rho=1/.95-1; %discount rate
r           = 0.03;
R = 1+ r;

J=9; %number of asset grid points
grid_size = [5;10;15;20;25;40;50;75;100;150;200;350;550;750;1000;1200;1500;2500;4000;5000;7500;9000];

true_I = 10000;

maxit= 1000;
crit = 1e-6;

%% Continuous Time
%% Reading True Values
%filename to read "true" policy function values obtained from a super-fine grid
fname= '../Data/cont_true_dist.txt';

%filename to read "true" agg consumption value obtained from a super-fine grid
aggconsname= '../Data/agg_c_cont.txt';

true_vals = csvread(aggconsname);
true_agg_c_cont = true_vals(1);
duration_c = true_vals(2);
agg_duration_c = true_vals(3);

% income grids for cont and disc time
igname_d1= '../Data/disc_igrid1.txt';
igname_d2= '../Data/disc_igrid2.txt';
igname_c= '../Data/cont_igrid.txt';

true_y_cont = csvread(igname_c);
true_y_disc1 = csvread(igname_d1);
true_y_disc2 = csvread(igname_d2);



%if the true value file does not exist, first run the super-fine grid file
if exist(fname, 'file')<2
    cont_true_dist
end

true_c_cont=csvread(fname);

%% Parameters

%ORNSTEIN-UHLENBECK PROCESS dlog(z) = -the*log(z)dt + sig2*dW
%STATIONARY DISTRIBUTION IS log(z) ~ N(0,Var) WHERE Var = sig2/(2*the)
Corr = 0.95; 
the = -log(Corr);
Var = 0.2^2;
sig2 = Var*2*the;
logzmean = 0;
zmean = logzmean + exp(Var/2); %MEAN OF LOG-NORMAL DISTRIBUTION N(0,Var)

%zmax = 99th percentile of log-normal distribution:
z_stan_99 = 2.33; z_stan_975 = 1.96; z_stan_95 = 1.6449; z_stan_999 = 3.09;
logzmax = z_stan_99*sqrt(Var);
zmax = exp(logzmax);
zmin = exp(-logzmax);
z = linspace(zmin,zmax,J);
z=z/zmean;
dz = (zmax-zmin)/(J-1);
dz2 = dz^2;

Delta = 10;

psi = 0.0;
amin = -psi*zmin/r; %borrowing constraint
amax=50;

% speeds and accuracy vectors
speed_cont=grid_size*0;
agg_speed_cont = grid_size*0;
agg_cons_cont = grid_size*0;
error_cont = grid_size*0;

%% Iterate on Grid Sizes

for s=1:length(grid_size)
    %% (non-uniform) Grid
    I = grid_size(s);
    coeff = 0; power=2;
    x = linspace(0,1,I).^power';
    a = amin+(amax-amin)*x;
    daf = ones(I,1);
    dab = ones(I,1);
    daf(1:I-1) = a(2:I)-a(1:I-1);
    dab(2:I) = a(2:I)-a(1:I-1);
    daf(I)=daf(I-1); dab(1)=dab(2);


    mu = (-the*log(z) + sig2/2).*z; %DRIFT (FROM ITO'S LEMMA)
    s2 = sig2.*z.^2; %VARIANCE (FROM ITO'S LEMMA)

    aa = a*ones(1,J);
    daaf = daf*ones(1,J);
    daab = dab*ones(1,J);
    zz = ones(I,1)*z;


    Vaf = zeros(I,J);
    Vab = zeros(I,J);
    Vzf = zeros(I,J);
    Vzb = zeros(I,J);
    Vzz = zeros(I,J);
    c = zeros(I,J);

    %CONSTRUCT MATRIX Aswitch SUMMARIZING EVOLUTION OF z
    yy = - s2/dz2 - mu/dz;
    chi =  s2/(2*dz2);
    zeta = mu/dz + s2/(2*dz2);

    %This will be the upperdiagonal of the B_switch
    updiag=zeros(I,1); %This is necessary because of the peculiar way spdiags is defined.
    for j=1:J
        updiag=[updiag;repmat(zeta(j),I,1)];
    end

    %This will be the center diagonal of the B_switch
    centdiag=repmat(chi(1)+yy(1),I,1);
    for j=2:J-1
        centdiag=[centdiag;repmat(yy(j),I,1)];
    end
    centdiag=[centdiag;repmat(yy(J)+zeta(J),I,1)];

    %This will be the lower diagonal of the B_switch
    lowdiag=repmat(chi(2),I,1);
    for j=3:J
        lowdiag=[lowdiag;repmat(chi(j),I,1)];
    end

    %Add up the upper, center, and lower diagonal into a sparse matrix
    Aswitch=spdiags(centdiag,0,I*J,I*J)+spdiags(lowdiag,-I,I*J,I*J)+spdiags(updiag,I,I*J,I*J);

    %% Initial Guess
    v0 = (zz + r.*aa).^(1-risk_aver)/(1-risk_aver)/rho;
    v = v0;

    %% Iteration
    tic;
    for n=1:maxit
        V = v;
        % forward difference
        Vaf(1:I-1,:) = (V(2:I,:)-V(1:I-1,:))./(aa(2:I,:) - aa(1:I-1,:));
        Vaf(I,:) = (max(z + r.*amax,10^(-10))).^(-risk_aver); %will never be used, but impose state constraint a<=amax just in case
        % backward difference
        Vab(2:I,:) = (V(2:I,:)-V(1:I-1,:))./(aa(2:I,:) - aa(1:I-1,:));
        Vab(1,:) = (max(z + r.*amin,10^(-10))).^(-risk_aver); %state constraint boundary condition

        I_concave = Vab > Vaf; %indicator whether value function is concave (problems arise if this is not the case)

        %consumption and savings with forward difference
        cf = max(Vaf,10^(-10)).^(-1/risk_aver);
        sf = zz + r.*aa - cf;
        %consumption and savings with backward difference
        cb = max(Vab,10^(-10)).^(-1/risk_aver);
        sb = zz + r.*aa - cb;
        %consumption and derivative of value function at steady state
        c0 = zz + r.*aa;

        % dV_upwind makes a choice of forward or backward differences based on
        % the sign of the drift
        If = sf > 0; %positive drift --> forward difference
        Ib = sb < 0; %negative drift --> backward difference
        I0 = (1-If-Ib); %at steady state
        %STATE CONSTRAINT at amin: USE BOUNDARY CONDITION UNLESS sf > 0:
        %already taken care of automatically

        c = cf.*If + cb.*Ib + c0.*I0;
        u = c.^(1-risk_aver)/(1-risk_aver);

        %CONSTRUCT MATRIX A
        X = -min(sb,0)./daab;
        Y = -max(sf,0)./daaf + min(sb,0)./daab;
        Z = max(sf,0)./daaf;

        updia=0; %This is needed because of the peculiarity of spdiags.
        for j=1:J
            updia=[updia;Z(1:I-1,j);0];
        end

        centdiag=reshape(Y,I*J,1);

        lowdia=X(2:I,1);
        for j=2:J
            lowdia=[lowdia;0;X(2:I,j)];
        end

        AA =spdiags(centdiag,0,I*J,I*J)+spdiags([updia;0],1,I*J,I*J)+spdiags([lowdia;0],-1,I*J,I*J);

        A = AA + Aswitch;
        B = (1/Delta + rho)*speye(I*J) - A;

        u_stacked = reshape(u,I*J,1);
        V_stacked = reshape(V,I*J,1);

        b = u_stacked + V_stacked/Delta;

        V_stacked = B\b; %SOLVE SYSTEM OF EQUATIONS

        V = reshape(V_stacked,I,J);

        Vchange = V - v;
        v = V;

        dist(n) = max(max(abs(Vchange)));
        %disp(['Iteration no. ' int2str(n), ' convergence criterion ' num2str(dist(n))]);
        
        if dist(n)<crit
            %disp('Value Function Converged, Iteration = ')
            %disp(n)
            break
        end
    end
    duration=toc;


    %% FOKKER-PLANCK EQUATION %
    %WORK WITH RESCALED DENSITY \tilde{g} BELOW
    tic;
    da_tilde = 0.5*(dab + daf);
    da_tilde(1) = 0.5*daf(1); da_tilde(I) = 0.5*dab(I);
    da_stacked = repmat(da_tilde,J,1);
    grid_diag = spdiags(da_stacked,0,I*J,I*J);

    AT = A';
    b = zeros(I*J,1);

    %need to fix one value, otherwise matrix is singular
    i_fix = 1;
    b(i_fix)=.1;
    row = [zeros(1,i_fix-1),1,zeros(1,I*J-i_fix)];
    AT(i_fix,:) = row;

    %Solve linear system
    g_tilde = AT\b;

    %rescale \tilde{g} so that it sums to 1
    g_sum = g_tilde'*ones(I*J,1);
    g_tilde = g_tilde./g_sum;

    gg = grid_diag\g_tilde; %convert from \tilde{g} to g

    g = reshape(gg,I,J);

    c_stacked = reshape(c,I*J,1);
    
    duration_dist = toc;
    %% Aggregate Consumption
    C = (c_stacked.*gg)'*da_stacked;
        
    speed_cont(s)=duration;
    agg_cons_cont(s)= C;
    agg_speed_cont(s) = duration + duration_dist;
    
    %% interpolate c_s for a grid of size 10000/true_I
    c_s = zeros(true_I,J);
    x_s = linspace(0,1,true_I).^power';
    a_s = amin+(amax-amin)*x_s;
    
    for jj=1:J
        for aa=1:true_I
            c_s(aa,jj) = lininterp1(a, c(:,jj), a_s(aa));
        end
    end
    
    error_cont(s)=mean(mean(abs(c_s-true_c_cont)./true_c_cont))*100;
            
end

error_aggc_cont = abs(agg_cons_cont-true_agg_c_cont)/true_agg_c_cont*100;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Discrete time
%% Reading True Values
%filename to store "true" values obtained from a super-fine grid
fname= '../Data/disc_true_dist.txt';

%filename to read "true" agg consumption value obtained from a super-fine grid
aggconsname= '../Data/agg_c_disc.txt';

true_vals = csvread(aggconsname);
true_agg_c_disc = true_vals(1);
duration_d = true_vals(2);
agg_duration_d = true_vals(3);

%if the ture value file does not exist, first run the super-fine grid file
if exist(fname, 'file')<2
    disc_true_disc
end

true_c_disc=csvread(fname);

%% PARAMETERS
% income risk: AR(1) + IID in logs
nyP         = J; % persistent component
rho_logyP   = 0.95;
sd_logyP    = 0.01;

nyT         = 5; %transitory component (not a state variable)
sd_logyT   = sqrt(0.04*(1-rho_logyP^2));

% cash on hand grid
xmax        = 50; 
xgrid_par   = 0.5; %1 for linear, 0 for L-shaped
borrow_lim  = 0;

% computation

Nsim = 50000;
Tsim = 500;

% DRAW RANDOM NUMBERS
rng(2020);
yPrand = rand(Nsim,Tsim);
yTrand = randn(Nsim,Tsim);

% speeds and accuracy vectors
speed_disc=grid_size*0;
agg_speed_disc = grid_size*0;
agg_cons_disc = grid_size*0;
error_disc = grid_size*0;

%% Iterate on Grid Sizes
for s=1:length(grid_size)
    %% SET UP GRIDS
    na=grid_size(s);
    
    %persistent income
    [logyPgrid, yPtrans, yPdist] = rouwenhorst(nyP, -0.5*sd_logyP^2, sd_logyP, rho_logyP);

    yPgrid = exp(logyPgrid);
    yPcumdist = cumsum(yPdist);
    yPcumtrans = cumsum(yPtrans,2);

    % transitory income: disretize normal distribution
    if nyT>1
        width = fzero(@(x)discrete_normal(nyT,-0.5*sd_logyT^2 ,sd_logyT ,x),2);
        [temp,logyTgrid,yTdist] = discrete_normal(nyT,-0.5*sd_logyT^2 ,sd_logyT ,width);

    elseif nyT==1
        logyTgrid = 0;
        yTdist = 1;
    end
    yTgrid = exp(logyTgrid);

    %gross labor income grid;
    ygrid = zeros(nyP,nyT);
    ydist= zeros(nyP,nyT);
    for iyP = 1:nyP
        for iyT= 1:nyT
            ygrid(iyP,iyT) = yPgrid(iyP)*yTgrid(iyT);
            ydist(iyP,iyT) = yPdist(iyP)*yTdist(iyT);
        end
    end    

    
    %% ASSET GRIDS
    xgrid = zeros(na,nyP);
    % cash on hand grids: different min points for each value of (iyP)
    for iyP = 1:nyP
        xgrid(:,iyP) = linspace(0,1,na)'.^(1/xgrid_par);
        xgrid(:,iyP) = borrow_lim + min(ygrid(iyP,:)) + (xmax-borrow_lim).*xgrid(:,iyP);
    end

    % savings grid for EGP
    ns = na;
    sgrid = linspace(0,1,na)';
    sgrid = sgrid.^(1/xgrid_par);
    sgrid = borrow_lim + (xmax-borrow_lim).*sgrid;

    %% UTILITY FUNCTION

    if risk_aver==1
        u = @(c)log(c);
    else    
        u = @(c)(c.^(1-risk_aver)-1)./(1-risk_aver);
    end    

    u1 = @(c) c.^(-risk_aver);
    u1inv = @(u) u.^(-1./risk_aver);

    %% INITIALIZE CONSUMPTION FUNCTION
    conguess = r.*xgrid;

    %% ITERATE ON EULER EQUATION WITH ENDOGENOUS GRID POINTS

    con = conguess;

    iter = 0;
    cdiff = 1000;
    muc1 = zeros(na,nyP);
    con1 = zeros(na,nyP);
    cash1 = zeros(na,nyP);
    
    tic;
    while iter <= maxit && cdiff>crit
        iter = iter + 1;
        sav = zeros(na,nyP);
        emuc= zeros(ns,nyP);

        conlast = con;
        muc  = u1(conlast);   %muc on grid for x' 

        %create interpolating function for each yP'
        for iy = 1:nyP
            coninterp{iy} = griddedInterpolant(xgrid(:,iy),conlast(:,iy),'linear');
        end


        % loop over current persistent income
        for iy = 1:nyP

            %loop over future income realizations    
            for iyP2 = 1:nyP
                for iyT2 = 1:nyT
                    emuc(:,iy) = emuc(:,iy) + u1(coninterp{iyP2}(R.*sgrid(:) + yPgrid(iyP2)*yTgrid(iyT2))) * yPtrans(iy,iyP2) * yTdist(iyT2);
                end
            end

            muc1(:,iy) = beta.*R.*emuc(:,iy);
            con1(:,iy) = u1inv(muc1(:,iy));
            cash1(:,iy) = con1(:,iy) + sgrid;


            % loop over current period cash on hand
            for ix = 1:na 
                if xgrid(ix,iy)<cash1(1,iy) %borrowing constraint binds
                    sav(ix,iy) = borrow_lim;
                else %borrowing constraint does not bind;
                    sav(ix,iy) = lininterp1(cash1(:,iy),sgrid,xgrid(ix,iy));
                end                
            end
            con(:,iy) = xgrid(:,iy) - sav(:,iy);
        end   

        cdiff = max(max(abs(con-conlast)));
        %disp(['Iteration no. ' int2str(iter), ' max con fn diff is ' num2str(cdiff)]);
    end    
    duration=toc;

    %% SIMULATE
    tic;
    yPindsim = zeros(Nsim,Tsim);
    logyTsim = zeros(Nsim,Tsim);
    logyPsim = zeros(Nsim,Tsim);
    ysim = zeros(Nsim,Tsim);    
    xsim = zeros(Nsim,Tsim);
    ssim = zeros(Nsim,Tsim);

    %create interpolating function
    for iy = 1:nyP
        savinterp{iy} = griddedInterpolant(xgrid(:,iy),sav(:,iy),'linear');
    end

    %initialize permanent income;
    it = 1;
    yPindsim(yPrand(:,it)<= yPcumdist(1),it) = 1;
    for iy = 2:nyP
        yPindsim(yPrand(:,it)> yPcumdist(iy-1) & yPrand(:,it)<=yPcumdist(iy),it) = iy;
    end

    %loop over time periods
    for it = 1:Tsim
        if mod(it,100) ==0
            %disp([' Simulating, time period ' int2str(it)]);
        end

        %permanent income realization
        if it > 1
            yPindsim(yPrand(:,it)<= yPcumtrans( yPindsim(:,it-1),1),it) = 1;
            for iy = 2:nyP
                yPindsim(yPrand(:,it)> yPcumtrans(yPindsim(:,it-1),iy-1) & yPrand(:,it)<=yPcumtrans(yPindsim(:,it-1),iy),it) = iy;
            end
        end
        logyPsim(:,it) = logyPgrid(yPindsim(:,it));

        %transitory income realization
        logyTsim(:,it) = - 0.5*sd_logyT^2 + yTrand(:,it).*sd_logyT;

        ysim(:,it) = exp(logyPsim(:,it) + logyTsim(:,it));

        %update cash on hand
        if it>1
            xsim(:,it) = R.*ssim(:,it-1) + ysim(:,it);
        end

        %savings choice
        if it<Tsim        
            for iy = 1:nyP
                ssim(yPindsim(:,it)==iy,it) = savinterp{iy}(xsim(yPindsim(:,it)==iy,it));
            end
        end
        ssim(ssim<borrow_lim) = borrow_lim;


    end

    %convert to assets
    asim = (xsim - ysim)./R;

    %consumption
    csim = xsim - ssim;
    
    duration_dist = toc;
    %% Aggregate consumption
    C = mean(csim(:,Tsim));
    
    speed_disc(s)=duration;
    agg_cons_disc(s)= C;
    agg_speed_disc(s) = duration + duration_dist;
    %% interpolate c_s for a grid of size 10000/true_I
    c_s = zeros(true_I,J);
    x_s = linspace(0,1,true_I).^power';
    a_s = amin+(amax-amin)*x_s;
    
    for jj=1:J
        for aa=1:true_I
            c_s(aa,jj) = lininterp1(sgrid, con(:,jj), a_s(aa));
        end
    end
    
    error_disc(s)=mean(mean(abs(c_s-true_c_disc)./true_c_disc))*100;
     
end

error_aggc_disc = abs(agg_cons_disc-true_agg_c_disc)/true_agg_c_disc*100;

%% Write data to .m files
accuracy_gain = error_disc ./ error_cont;
accuracy_gain_agg = error_aggc_disc./error_aggc_cont;
speed_gain = speed_disc./speed_cont;
agg_speed_gain = agg_speed_disc ./ agg_speed_cont;

plot_data = [grid_size'; accuracy_gain'; error_cont'; error_disc'; error_aggc_cont'; error_aggc_disc';
     speed_cont'; speed_disc'; speed_gain'; agg_speed_cont'; agg_speed_disc'];
 
fname= 'plot_data.txt';
p = '../Data/';
fname=strcat(p, fname);
csvwrite(fname, plot_data);

%% Write data to Tex Table
idxes = [2, 5, 7, 9, 15];

M_p = [grid_size,speed_cont, error_cont, speed_disc, error_disc];
M_p = M_p(idxes,:);

last_row_p = [true_I, duration_c, 0, duration_d, 0];

input_p.data = [M_p;last_row_p];
input_p.dataFormat = {'%d',1,'%.2f',4};


M_a = [grid_size, agg_speed_cont, error_aggc_cont, agg_speed_disc, error_aggc_disc];
M_a = M_a(idxes,:);

last_row_a = [true_I, agg_duration_c, 0, agg_duration_d, 0];

input_a.data = [M_a;last_row_a];
input_a.dataFormat = {'%d',1,'%.2f',4};

latex = latexTable2(input_p, input_a);

p = '../Data/';
dname = strcat(p, 'Tex');
if ~exist(dname, 'dir')
   mkdir(dname)
end

fname=fullfile(dname, 'acctable.tex');

fileID = fopen(fname,'w');
fprintf(fileID, '%s\n',latex{:});
fclose(fileID);