function [M1,nu,s1der,c1,c2,MPC1,MPC2,MPC1_PV] = partialeq_subroutine(params,grid_params)

tic

s=params(1);
r=params(2);
rho = params(3);
z1 = params(4);
z2=params(5);
la1=params(6);
la2=params(7);
tau=params(8);

amin = grid_params(1);
amax = grid_params(2);
I=grid_params(3);

la = [la1,la2];
z = [z1,z2];

a = linspace(amin,amax,I)';
da = (amax-amin)/(I-1);

aa = [a,a];
zz = ones(I,1)*z;


maxit= 100;
crit = 10^(-6);
Delta = 1000;

dVf = zeros(I,2);
dVb = zeros(I,2);
c = zeros(I,2);

Aswitch = [-speye(I)*la(1),speye(I)*la(1);speye(I)*la(2),-speye(I)*la(2)];

%INITIAL GUESS
v0(:,1) = (z(1) + r.*a).^(1-s)/(1-s)/rho;
v0(:,2) = (z(2) + r.*a).^(1-s)/(1-s)/rho;

v = v0;

for n=1:maxit
    V = v;
    V_n(:,:,n)=V;
    % forward difference
    dVf(1:I-1,:) = (V(2:I,:)-V(1:I-1,:))/da;
    dVf(I,:) = (z + r.*amax).^(-s); %will never be used, but impose state constraint a<=amax just in case
    % backward difference
    dVb(2:I,:) = (V(2:I,:)-V(1:I-1,:))/da;
    dVb(1,:) = (z + r.*amin).^(-s); %state constraint boundary condition
       
    %consumption and savings with forward difference
    cf = dVf.^(-1/s);
    ssf = zz + r.*aa - cf;
    %consumption and savings with backward difference
    cb = dVb.^(-1/s);
    ssb = zz + r.*aa - cb;
    %consumption and derivative of value function at steady state
    c0 = zz + r.*aa;
    dV0 = c0.^(-s);
    
    % dV_upwind makes a choice of forward or backward differences based on
    % the sign of the drift    
    If = ssf > 0; %positive drift --> forward difference
    Ib = ssb < 0; %negative drift --> backward difference
    I0 = (1-If-Ib); %at steady state
    %make sure backward difference is used at amax
    %Ib(I,:) = 1; If(I,:) = 0;
    %STATE CONSTRAINT at amin: USE BOUNDARY CONDITION UNLESS muf > 0:
    %already taken care of automatically
    
    dV_Upwind = dVf.*If + dVb.*Ib + dV0.*I0; %important to include third term
    c = dV_Upwind.^(-1/s);
    u = c.^(1-s)/(1-s);
    
    %CONSTRUCT MATRIX
    X = - min(ssb,0)/da;
    Y = - max(ssf,0)/da + min(ssb,0)/da;
    Z = max(ssf,0)/da;
    
    A1=spdiags(Y(:,1),0,I,I)+spdiags(X(2:I,1),-1,I,I)+spdiags([0;Z(1:I-1,1)],1,I,I);
    A2=spdiags(Y(:,2),0,I,I)+spdiags(X(2:I,2),-1,I,I)+spdiags([0;Z(1:I-1,2)],1,I,I);

    A = [A1,sparse(I,I);sparse(I,I),A2] + Aswitch;
    B = (1/Delta + rho)*speye(2*I) - A;
    
    u_stacked = [u(:,1);u(:,2)];
    V_stacked = [V(:,1);V(:,2)];
    
    b = u_stacked + V_stacked/Delta;
    V_stacked = B\b; %SOLVE SYSTEM OF EQUATIONS
    
    V = [V_stacked(1:I),V_stacked(I+1:2*I)];
    
    Vchange = V - v;
    v = V;

    dist(n) = max(max(abs(Vchange)));
    if dist(n)<crit
        disp('Value Function Converged, Iteration = ')
        disp(n)
        break
    end
end
toc;

%%%%%%%%%%%%%%%%%%%%%%%%%%
% FOKKER-PLANCK EQUATION %
%%%%%%%%%%%%%%%%%%%%%%%%%%
AT = A';
b = zeros(2*I,1);

%need to fix one value, otherwise matrix is singular
i_fix = 1;
b(i_fix)=.1;
row = [zeros(1,i_fix-1),1,zeros(1,2*I-i_fix)];
AT(i_fix,:) = row;

%Solve linear system
gg = AT\b;
g_sum = gg'*ones(2*I,1)*da;
gg = gg./g_sum;

g = [gg(1:I),gg(I+1:2*I)];

check1 = g(:,1)'*ones(I,1)*da;
check2 = g(:,2)'*ones(I,1)*da;

adot = zz + r.*aa - c;

S = g(:,1)'*a*da + g(:,2)'*a*da;

M1 = g(1,1)*da;

cmin1 = c(1,1);
cmin2 = c(1,2);
c1 = c(:,1);
c2 = c(:,2);

nu = sqrt(2*cmin1/s*(rho - r + la1*(1 - (cmin2/cmin1)^(-s))));
s1der = -(adot(2,1)-adot(1,1))/da;


%%%%%%%%%%%%%%%%%%
% CALCULATE MPCs %
%%%%%%%%%%%%%%%%%%
N = 21;
dt = tau/(N-1);

C_stacked = zeros(2*I,N);
ctilde = zeros(2*I,1);

C_stacked(:,N)=0;

B = (1/dt)*speye(2*I) - A;
c_stacked = reshape(c,2*I,1);

for n=N-1:-1:1
    vec = c_stacked + C_stacked(:,n+1)/dt;
    C_stacked(:,n) = B\vec;
end

C = reshape(C_stacked(:,1),I,2);
MPC = (C(2:I,:)-C(1:I-1,:))/da;

%PRESENT VALUE MPC
C_stacked_PV = (rho*speye(2*I)-A)\c_stacked;
C_PV = reshape(C_stacked_PV,I,2);
MPC_PV = (C_PV(2:I,:)-C_PV(1:I-1,:))/da;

MPC1 = MPC(:,1);
MPC2 = MPC(:,2);
MPC1_PV = MPC_PV(:,1);

